#include "at45db0x1.h"

#include <stdlib.h>
#include <defines.h>

#ifdef HAVE_CONFIG_H
#if AT45_USE_SPI_NUMBER == 0
#include <spi0.h>
#define AT45_write			spi0_write
#define AT45_dummyWrite 	spi0_dummyWrite
#define AT45_writeBlock		spi0_writeBlock
#define AT45_readBlock		spi0_readBlock
#define AT45_enable			spi0_enable
#define AT45_disable		spi0_disable		
#elif AT45_USE_SPI_NUMBER == 1
#include <spi1.h>
#define AT45_write			spi1_write
#define AT45_dummyWrite 	spi1_dummyWrite
#define AT45_writeBlock		spi1_writeBlock
#define AT45_readBlock		spi1_readBlock
#define AT45_enable			spi1_enable
#define AT45_disable		spi1_disable		
#endif
#endif

#define CMD_READ_STATUS		0xD7
#define CMD_READ_ARRAY		0xE8
#define CMD_BUFFER_TO_PAGE	0x83
#define CMD_PAGE_TO_BUFFER	0x53
#define CMD_BUFFER_READ		0xD4
#define CMD_BUFFER_WRITE	0x84

static enum { WriteStateIdle, WriteStatePageToBuffer, WriteStateWriteToBuffer, WriteStateBufferToPage } seGlblWriteState = WriteStateIdle;
static enum { ModeIdle, ModeRead, ModeWrite } seGlblMode;
static uint16_t su16GlblPage;
static uint16_t su16GlblOffset;
static uint16_t su16GlblLength;
static uint32_t su32GlblAddr;
static union {
	uint8_t* readData;
	const uint8_t* writeData;
} sGlblBuffer;

static bool writeBlockPeriodic(void);

static bool isBusy(void);
static bool isBusy()
{
	uint8_t status;
	
	AT45_enable();
	AT45_write(CMD_READ_STATUS);
	status = AT45_write(0);
	AT45_disable();
	
	return !(status & _BV(7));
}

void at45_busyWait()
{	
	while (isBusy()) { /* empty */ }
}

bool at45_isBusy()
{
	return (seGlblMode == ModeWrite && su16GlblLength > 0) || isBusy();
}

bool at45_isReading()
{
	return seGlblMode == ModeRead;
}

void at45_pageToBuffer(uint16_t page)
{
	AT45_enable();
	AT45_write(CMD_PAGE_TO_BUFFER);
	AT45_write(page >> (16 - AT45_PAGEBITS));
	AT45_write(page << (AT45_PAGEBITS - 8));
	AT45_write(0x00);
	AT45_disable();
}

void at45_readFromBuffer(uint16_t offset, uint8_t* dest, uint16_t length)
{
	AT45_enable();
	AT45_write(CMD_BUFFER_READ);
	AT45_write(0x00);
	AT45_write(offset >> 8);
	AT45_write(offset);
	AT45_write(0x00);
	
	while (length) {
		*dest = AT45_write(0x00);
		dest++;
		length--;
	}
	AT45_disable();
}

void at45_bufferToPage(uint16_t page)
{
	AT45_enable();
	AT45_write(CMD_BUFFER_TO_PAGE);
	AT45_write(page >> (16 - AT45_PAGEBITS));
	AT45_write(page << (AT45_PAGEBITS - 8));
	AT45_write(0x00);
	AT45_disable();
}

void at45_writeToBuffer(uint16_t offset, const uint8_t* src, uint16_t length)
{
	AT45_enable();
	AT45_write(CMD_BUFFER_WRITE);
	AT45_write(0x00);
	AT45_write(offset >> 8);
	AT45_write(offset);
	
	while (length) {
		AT45_write(*src);
		src++;
		length--;
	}
	AT45_disable();
}

void at45_writeBlock(uint32_t addr, const uint8_t* src, uint16_t length)
{	
	su16GlblPage     = addr / (uint16_t) AT45_PAGESIZE;
	su16GlblOffset   = addr % (uint16_t) AT45_PAGESIZE;
	su16GlblLength   = length;
	sGlblBuffer.writeData   = src;
	seGlblWriteState = WriteStateIdle;
	seGlblMode       = ModeWrite;
}

static bool writeBlockPeriodic()
{
	static uint16_t su16Bytes;
	
	if (su16GlblLength > 0) {
		switch (seGlblWriteState) {
			case WriteStateIdle:
				if (isBusy()) return false;
				su16Bytes = AT45_PAGESIZE - su16GlblOffset;
				if (su16Bytes > su16GlblLength) su16Bytes = su16GlblLength;
				at45_pageToBuffer(su16GlblPage);
				seGlblWriteState = WriteStatePageToBuffer;
				break;
			case WriteStatePageToBuffer:
				if (isBusy()) return false;
				at45_writeToBuffer(su16GlblOffset, sGlblBuffer.writeData, su16Bytes);
				seGlblWriteState = WriteStateWriteToBuffer;
				break;
			case WriteStateWriteToBuffer:
				if (isBusy()) return false;
				at45_bufferToPage(su16GlblPage);
				seGlblWriteState = WriteStateBufferToPage;
				break;
			case WriteStateBufferToPage:
				if (isBusy()) return false;
				su16GlblLength -= su16Bytes;
				sGlblBuffer.writeData += su16Bytes;
				su16GlblPage++;
				su16GlblOffset = 0;
				seGlblWriteState = WriteStateIdle;
				break;
		}
	}
	else {
		return true;
	}
	
	return false;
}

void at45_readBlock(uint32_t addr, uint8_t* dest, uint16_t length)
{
	su32GlblAddr     = addr;
	su16GlblLength   = length;
	sGlblBuffer.readData = dest;
	seGlblMode       = ModeRead;
}

void at45_readBlock_sync(uint32_t addr, uint8_t* dest, uint16_t length)
{
	uint16_t page;
	uint16_t offset;

	page   = addr / AT45_PAGESIZE;
	offset = addr % AT45_PAGESIZE;
	
	AT45_enable();
	AT45_write(CMD_READ_ARRAY); /* Continuous array read */
	AT45_write(page >> (16 - AT45_PAGEBITS));
	AT45_write((page << (AT45_PAGEBITS - 8)) | (offset >> 8));
	AT45_write(offset);
	AT45_dummyWrite(4);
	AT45_readBlock(dest, length);
	AT45_disable();
}

void at45_init()
{
	AT45_disable();
}

bool at45_doSomeWork()
{
	bool w = true;
	
	switch (seGlblMode)
	{
		case ModeRead:
			at45_readBlock_sync(su32GlblAddr, sGlblBuffer.readData, su16GlblLength);
			seGlblMode = ModeIdle;
			break;
		case ModeWrite:
			w = writeBlockPeriodic();
			if (w) seGlblMode = ModeIdle;
			break;
		default:
			break;
	}
	
	return w;
}
