/* Driver for Atmel DataFlash AT45DB0x1 series.
 * 
 * History:
 * 20070327 let - Initial version
 * 
 */
#ifndef AT45DB0X1_H_
#define AT45DB0X1_H_

#include <stdint.h>
#include <stdbool.h>

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
/* Specify which SPI interface to use {0,1} */
#define AT45_USE_SPI_NUMBER   0

/* Page size used by DataFlash */
#define AT45_PAGESIZE   264

/* Number of address bits used by DataFlash */
#define AT45_PAGEBITS	  9
#endif

#ifdef __cplusplus
	extern "C" {
#endif

/* Write a block of data.
 * The address is absolute and writing across page boundaries is allowed.
 * This function is asynchronous and returns immediately. Call at45_doSomeWork()
 * periodically to write the block. Use at45_isBusy() or at45_busyWait() if you
 * need to know when your data have been written.
 */ 
void at45_writeBlock(uint32_t addr, const uint8_t* src, uint16_t length);

/* Read a block of data.
 * The address is absolute and reading across page boundaries is allowed.
 * This function is asynchronous and returns immediately. Call at45_doSomeWork()
 * periodically to read the block. Use at45_isBusy() or at45_busyWait() if you
 * need to know when your data have been read.
 */
void at45_readBlock(uint32_t addr, uint8_t* dest, uint16_t length);

/* Read a block of data.
 * The address is absolute and reading across page boundaries is allowed.
 * This function is synchronous and returns as soon as your data have
 * been read.
 */
void at45_readBlock_sync(uint32_t addr, uint8_t* dest, uint16_t length);

/* Copy a page from main (flash) memory to the internal RAM.
 */ 
void at45_pageToBuffer(uint16_t page);

/* Read data from the internal RAM.
 * The offset specifies the first byte within the internal RAM to be read.
 */
void at45_readFromBuffer(uint16_t offset, uint8_t* dest, uint16_t length);

/* Write data to the internal buffer.
 * The offset specifies the first byte within the internal RAM to be written to.
 */
void at45_writeToBuffer(uint16_t offset, const uint8_t* src, uint16_t length);

/* Write contents of the internal RAM to a page in the main (flash) memory.
 */
void at45_bufferToPage(uint16_t page);

/* Block while flash is busy.
 */
void at45_busyWait(void);

bool at45_isBusy(void);
bool at45_isReading(void);

void at45_init(void);

/* Asynchronous processing.
 */
bool at45_doSomeWork(void);

#ifdef __cplusplus
	}
#endif    

#endif /*AT45DB0X1_H_*/
